@@ -19,8 +19,8 @@ extern crate rustc_middle;
 mod build;
 mod check_unsafety;
 mod errors;
-mod lints;
-pub mod thir;
+pub mod lints;
+mod thir;
 
 use rustc_middle::query::Providers;
 
