@@ -9,8 +9,8 @@ use url::Url;
 use crate::core::PackageId;
 use crate::util::edit_distance;
 use crate::util::errors::CargoResult;
-use crate::util::interning::InternedString;
-use crate::util::{validate_package_name, IntoUrl, ToSemver};
+use crate::util::{validate_package_name, IntoUrl};
+use crate::util_semver::PartialVersion;
 
 /// Some or all of the data required to identify a package:
 ///
@@ -23,8 +23,8 @@ use crate::util::{validate_package_name, IntoUrl, ToSemver};
 /// sufficient to uniquely define a package ID.
 #[derive(Clone, PartialEq, Eq, Debug, Hash, Ord, PartialOrd)]
 pub struct PackageIdSpec {
-    name: InternedString,
-    version: Option<Version>,
+    name: String,
+    version: Option<PartialVersion>,
     url: Option<Url>,
 }
 
@@ -70,12 +70,12 @@ impl PackageIdSpec {
         let mut parts = spec.splitn(2, [':', '@']);
         let name = parts.next().unwrap();
         let version = match parts.next() {
-            Some(version) => Some(version.to_semver()?),
+            Some(version) => Some(version.parse::<PartialVersion>()?),
             None => None,
         };
         validate_package_name(name, "pkgid", "")?;
         Ok(PackageIdSpec {
-            name: InternedString::new(name),
+            name: String::from(name),
             version,
             url: None,
         })
@@ -94,12 +94,12 @@ impl PackageIdSpec {
         spec.query(i)
     }
 
-    /// Convert a `PackageId` to a `PackageIdSpec`, which will have both the `Version` and `Url`
+    /// Convert a `PackageId` to a `PackageIdSpec`, which will have both the `PartialVersion` and `Url`
     /// fields filled in.
     pub fn from_package_id(package_id: PackageId) -> PackageIdSpec {
         PackageIdSpec {
-            name: package_id.name(),
-            version: Some(package_id.version().clone()),
+            name: String::from(package_id.name().as_str()),
+            version: Some(package_id.version().clone().into()),
             url: Some(package_id.source_id().url().clone()),
         }
     }
@@ -123,25 +123,21 @@ impl PackageIdSpec {
                 )
             })?;
             match frag {
-                Some(fragment) => {
-                    let mut parts = fragment.splitn(2, [':', '@']);
-                    let name_or_version = parts.next().unwrap();
-                    match parts.next() {
-                        Some(part) => {
-                            let version = part.to_semver()?;
-                            (InternedString::new(name_or_version), Some(version))
-                        }
-                        None => {
-                            if name_or_version.chars().next().unwrap().is_alphabetic() {
-                                (InternedString::new(name_or_version), None)
-                            } else {
-                                let version = name_or_version.to_semver()?;
-                                (InternedString::new(path_name), Some(version))
-                            }
+                Some(fragment) => match fragment.split_once([':', '@']) {
+                    Some((name, part)) => {
+                        let version = part.parse::<PartialVersion>()?;
+                        (String::from(name), Some(version))
+                    }
+                    None => {
+                        if fragment.chars().next().unwrap().is_alphabetic() {
+                            (String::from(fragment.as_str()), None)
+                        } else {
+                            let version = fragment.parse::<PartialVersion>()?;
+                            (String::from(path_name), Some(version))
                         }
                     }
-                }
-                None => (InternedString::new(path_name), None),
+                },
+                None => (String::from(path_name), None),
             }
         };
         Ok(PackageIdSpec {
@@ -151,11 +147,16 @@ impl PackageIdSpec {
         })
     }
 
-    pub fn name(&self) -> InternedString {
-        self.name
+    pub fn name(&self) -> &str {
+        self.name.as_str()
     }
 
-    pub fn version(&self) -> Option<&Version> {
+    /// Full `semver::Version`, if present
+    pub fn version(&self) -> Option<Version> {
+        self.version.as_ref().and_then(|v| v.to_version())
+    }
+
+    pub fn partial_version(&self) -> Option<&PartialVersion> {
         self.version.as_ref()
     }
 
@@ -169,20 +170,23 @@ impl PackageIdSpec {
 
     /// Checks whether the given `PackageId` matches the `PackageIdSpec`.
     pub fn matches(&self, package_id: PackageId) -> bool {
-        if self.name() != package_id.name() {
+        if self.name() != package_id.name().as_str() {
             return false;
         }
 
         if let Some(ref v) = self.version {
-            if v != package_id.version() {
+            if !v.matches(package_id.version()) {
                 return false;
             }
         }
 
-        match self.url {
-            Some(ref u) => u == package_id.source_id().url(),
-            None => true,
+        if let Some(u) = &self.url {
+            if u != package_id.source_id().url() {
+                return false;
+            }
         }
+
+        true
     }
 
     /// Checks a list of `PackageId`s to find 1 that matches this `PackageIdSpec`. If 0, 2, or
@@ -193,55 +197,52 @@ impl PackageIdSpec {
     {
         let all_ids: Vec<_> = i.into_iter().collect();
         let mut ids = all_ids.iter().copied().filter(|&id| self.matches(id));
-        let ret = match ids.next() {
-            Some(id) => id,
-            None => {
-                let mut suggestion = String::new();
-                let try_spec = |spec: PackageIdSpec, suggestion: &mut String| {
-                    let try_matches: Vec<_> = all_ids
-                        .iter()
-                        .copied()
-                        .filter(|&id| spec.matches(id))
-                        .collect();
-                    if !try_matches.is_empty() {
-                        suggestion.push_str("\nDid you mean one of these?\n");
-                        minimize(suggestion, &try_matches, self);
-                    }
-                };
-                if self.url.is_some() {
-                    try_spec(
-                        PackageIdSpec {
-                            name: self.name,
-                            version: self.version.clone(),
-                            url: None,
-                        },
-                        &mut suggestion,
-                    );
+        let Some(ret) = ids.next() else {
+            let mut suggestion = String::new();
+            let try_spec = |spec: PackageIdSpec, suggestion: &mut String| {
+                let try_matches: Vec<_> = all_ids
+                    .iter()
+                    .copied()
+                    .filter(|&id| spec.matches(id))
+                    .collect();
+                if !try_matches.is_empty() {
+                    suggestion.push_str("\nDid you mean one of these?\n");
+                    minimize(suggestion, &try_matches, self);
                 }
-                if suggestion.is_empty() && self.version.is_some() {
-                    try_spec(
-                        PackageIdSpec {
-                            name: self.name,
-                            version: None,
-                            url: None,
-                        },
-                        &mut suggestion,
-                    );
-                }
-                if suggestion.is_empty() {
-                    suggestion.push_str(&edit_distance::closest_msg(
-                        &self.name,
-                        all_ids.iter(),
-                        |id| id.name().as_str(),
-                    ));
-                }
-
-                bail!(
-                    "package ID specification `{}` did not match any packages{}",
-                    self,
-                    suggestion
+            };
+            if self.url.is_some() {
+                try_spec(
+                    PackageIdSpec {
+                        name: self.name.clone(),
+                        version: self.version.clone(),
+                        url: None,
+                    },
+                    &mut suggestion,
                 );
             }
+            if suggestion.is_empty() && self.version.is_some() {
+                try_spec(
+                    PackageIdSpec {
+                        name: self.name.clone(),
+                        version: None,
+                        url: None,
+                    },
+                    &mut suggestion,
+                );
+            }
+            if suggestion.is_empty() {
+                suggestion.push_str(&edit_distance::closest_msg(
+                    &self.name,
+                    all_ids.iter(),
+                    |id| id.name().as_str(),
+                ));
+            }
+
+            bail!(
+                "package ID specification `{}` did not match any packages{}",
+                self,
+                suggestion
+            );
         };
         return match ids.next() {
             Some(other) => {
@@ -250,8 +251,8 @@ impl PackageIdSpec {
                      your project, and the specification \
                      `{}` is ambiguous.\n\
                      Please re-run this command \
-                     with `-p <spec>` where `<spec>` is one \
-                     of the following:",
+                     with one of the following \
+                     specifications:",
                     self.name(),
                     self
                 );
@@ -325,8 +326,6 @@ impl<'de> de::Deserialize<'de> for PackageIdSpec {
 mod tests {
     use super::PackageIdSpec;
     use crate::core::{PackageId, SourceId};
-    use crate::util::interning::InternedString;
-    use crate::util::ToSemver;
     use url::Url;
 
     #[test]
@@ -335,13 +334,16 @@ mod tests {
         fn ok(spec: &str, expected: PackageIdSpec, expected_rendered: &str) {
             let parsed = PackageIdSpec::parse(spec).unwrap();
             assert_eq!(parsed, expected);
-            assert_eq!(parsed.to_string(), expected_rendered);
+            let rendered = parsed.to_string();
+            assert_eq!(rendered, expected_rendered);
+            let reparsed = PackageIdSpec::parse(&rendered).unwrap();
+            assert_eq!(reparsed, expected);
         }
 
         ok(
             "https://crates.io/foo",
             PackageIdSpec {
-                name: InternedString::new("foo"),
+                name: String::from("foo"),
                 version: None,
                 url: Some(Url::parse("https://crates.io/foo").unwrap()),
             },
@@ -350,17 +352,26 @@ mod tests {
         ok(
             "https://crates.io/foo#1.2.3",
             PackageIdSpec {
-                name: InternedString::new("foo"),
-                version: Some("1.2.3".to_semver().unwrap()),
+                name: String::from("foo"),
+                version: Some("1.2.3".parse().unwrap()),
                 url: Some(Url::parse("https://crates.io/foo").unwrap()),
             },
             "https://crates.io/foo#1.2.3",
         );
+        ok(
+            "https://crates.io/foo#1.2",
+            PackageIdSpec {
+                name: String::from("foo"),
+                version: Some("1.2".parse().unwrap()),
+                url: Some(Url::parse("https://crates.io/foo").unwrap()),
+            },
+            "https://crates.io/foo#1.2",
+        );
         ok(
             "https://crates.io/foo#bar:1.2.3",
             PackageIdSpec {
-                name: InternedString::new("bar"),
-                version: Some("1.2.3".to_semver().unwrap()),
+                name: String::from("bar"),
+                version: Some("1.2.3".parse().unwrap()),
                 url: Some(Url::parse("https://crates.io/foo").unwrap()),
             },
             "https://crates.io/foo#bar@1.2.3",
@@ -368,16 +379,25 @@ mod tests {
         ok(
             "https://crates.io/foo#bar@1.2.3",
             PackageIdSpec {
-                name: InternedString::new("bar"),
-                version: Some("1.2.3".to_semver().unwrap()),
+                name: String::from("bar"),
+                version: Some("1.2.3".parse().unwrap()),
                 url: Some(Url::parse("https://crates.io/foo").unwrap()),
             },
             "https://crates.io/foo#bar@1.2.3",
         );
+        ok(
+            "https://crates.io/foo#bar@1.2",
+            PackageIdSpec {
+                name: String::from("bar"),
+                version: Some("1.2".parse().unwrap()),
+                url: Some(Url::parse("https://crates.io/foo").unwrap()),
+            },
+            "https://crates.io/foo#bar@1.2",
+        );
         ok(
             "foo",
             PackageIdSpec {
-                name: InternedString::new("foo"),
+                name: String::from("foo"),
                 version: None,
                 url: None,
             },
@@ -386,8 +406,8 @@ mod tests {
         ok(
             "foo:1.2.3",
             PackageIdSpec {
-                name: InternedString::new("foo"),
-                version: Some("1.2.3".to_semver().unwrap()),
+                name: String::from("foo"),
+                version: Some("1.2.3".parse().unwrap()),
                 url: None,
             },
             "foo@1.2.3",
@@ -395,22 +415,122 @@ mod tests {
         ok(
             "foo@1.2.3",
             PackageIdSpec {
-                name: InternedString::new("foo"),
-                version: Some("1.2.3".to_semver().unwrap()),
+                name: String::from("foo"),
+                version: Some("1.2.3".parse().unwrap()),
                 url: None,
             },
             "foo@1.2.3",
         );
+        ok(
+            "foo@1.2",
+            PackageIdSpec {
+                name: String::from("foo"),
+                version: Some("1.2".parse().unwrap()),
+                url: None,
+            },
+            "foo@1.2",
+        );
+
+        // pkgid-spec.md
+        ok(
+            "regex",
+            PackageIdSpec {
+                name: String::from("regex"),
+                version: None,
+                url: None,
+            },
+            "regex",
+        );
+        ok(
+            "regex@1.4",
+            PackageIdSpec {
+                name: String::from("regex"),
+                version: Some("1.4".parse().unwrap()),
+                url: None,
+            },
+            "regex@1.4",
+        );
+        ok(
+            "regex@1.4.3",
+            PackageIdSpec {
+                name: String::from("regex"),
+                version: Some("1.4.3".parse().unwrap()),
+                url: None,
+            },
+            "regex@1.4.3",
+        );
+        ok(
+            "https://github.com/rust-lang/crates.io-index#regex",
+            PackageIdSpec {
+                name: String::from("regex"),
+                version: None,
+                url: Some(Url::parse("https://github.com/rust-lang/crates.io-index").unwrap()),
+            },
+            "https://github.com/rust-lang/crates.io-index#regex",
+        );
+        ok(
+            "https://github.com/rust-lang/crates.io-index#regex@1.4.3",
+            PackageIdSpec {
+                name: String::from("regex"),
+                version: Some("1.4.3".parse().unwrap()),
+                url: Some(Url::parse("https://github.com/rust-lang/crates.io-index").unwrap()),
+            },
+            "https://github.com/rust-lang/crates.io-index#regex@1.4.3",
+        );
+        ok(
+            "https://github.com/rust-lang/cargo#0.52.0",
+            PackageIdSpec {
+                name: String::from("cargo"),
+                version: Some("0.52.0".parse().unwrap()),
+                url: Some(Url::parse("https://github.com/rust-lang/cargo").unwrap()),
+            },
+            "https://github.com/rust-lang/cargo#0.52.0",
+        );
+        ok(
+            "https://github.com/rust-lang/cargo#cargo-platform@0.1.2",
+            PackageIdSpec {
+                name: String::from("cargo-platform"),
+                version: Some("0.1.2".parse().unwrap()),
+                url: Some(Url::parse("https://github.com/rust-lang/cargo").unwrap()),
+            },
+            "https://github.com/rust-lang/cargo#cargo-platform@0.1.2",
+        );
+        ok(
+            "ssh://git@github.com/rust-lang/regex.git#regex@1.4.3",
+            PackageIdSpec {
+                name: String::from("regex"),
+                version: Some("1.4.3".parse().unwrap()),
+                url: Some(Url::parse("ssh://git@github.com/rust-lang/regex.git").unwrap()),
+            },
+            "ssh://git@github.com/rust-lang/regex.git#regex@1.4.3",
+        );
+        ok(
+            "file:///path/to/my/project/foo",
+            PackageIdSpec {
+                name: String::from("foo"),
+                version: None,
+                url: Some(Url::parse("file:///path/to/my/project/foo").unwrap()),
+            },
+            "file:///path/to/my/project/foo",
+        );
+        ok(
+            "file:///path/to/my/project/foo#1.1.8",
+            PackageIdSpec {
+                name: String::from("foo"),
+                version: Some("1.1.8".parse().unwrap()),
+                url: Some(Url::parse("file:///path/to/my/project/foo").unwrap()),
+            },
+            "file:///path/to/my/project/foo#1.1.8",
+        );
     }
 
     #[test]
     fn bad_parsing() {
         assert!(PackageIdSpec::parse("baz:").is_err());
         assert!(PackageIdSpec::parse("baz:*").is_err());
-        assert!(PackageIdSpec::parse("baz:1.0").is_err());
         assert!(PackageIdSpec::parse("baz@").is_err());
         assert!(PackageIdSpec::parse("baz@*").is_err());
-        assert!(PackageIdSpec::parse("baz@1.0").is_err());
+        assert!(PackageIdSpec::parse("baz@^1.0").is_err());
         assert!(PackageIdSpec::parse("https://baz:1.0").is_err());
         assert!(PackageIdSpec::parse("https://#baz:1.0").is_err());
     }
@@ -419,14 +539,56 @@ mod tests {
     fn matching() {
         let url = Url::parse("https://example.com").unwrap();
         let sid = SourceId::for_registry(&url).unwrap();
-        let foo = PackageId::new("foo", "1.2.3", sid).unwrap();
-        let bar = PackageId::new("bar", "1.2.3", sid).unwrap();
 
+        let foo = PackageId::new("foo", "1.2.3", sid).unwrap();
         assert!(PackageIdSpec::parse("foo").unwrap().matches(foo));
-        assert!(!PackageIdSpec::parse("foo").unwrap().matches(bar));
+        assert!(!PackageIdSpec::parse("bar").unwrap().matches(foo));
         assert!(PackageIdSpec::parse("foo:1.2.3").unwrap().matches(foo));
         assert!(!PackageIdSpec::parse("foo:1.2.2").unwrap().matches(foo));
         assert!(PackageIdSpec::parse("foo@1.2.3").unwrap().matches(foo));
         assert!(!PackageIdSpec::parse("foo@1.2.2").unwrap().matches(foo));
+        assert!(PackageIdSpec::parse("foo@1.2").unwrap().matches(foo));
+        assert!(PackageIdSpec::parse("https://example.com#foo@1.2")
+            .unwrap()
+            .matches(foo));
+        assert!(!PackageIdSpec::parse("https://bob.com#foo@1.2")
+            .unwrap()
+            .matches(foo));
+
+        let meta = PackageId::new("meta", "1.2.3+hello", sid).unwrap();
+        assert!(PackageIdSpec::parse("meta").unwrap().matches(meta));
+        assert!(PackageIdSpec::parse("meta@1").unwrap().matches(meta));
+        assert!(PackageIdSpec::parse("meta@1.2").unwrap().matches(meta));
+        assert!(PackageIdSpec::parse("meta@1.2.3").unwrap().matches(meta));
+        assert!(!PackageIdSpec::parse("meta@1.2.3-alpha.0")
+            .unwrap()
+            .matches(meta));
+        assert!(PackageIdSpec::parse("meta@1.2.3+hello")
+            .unwrap()
+            .matches(meta));
+        assert!(!PackageIdSpec::parse("meta@1.2.3+bye")
+            .unwrap()
+            .matches(meta));
+
+        let pre = PackageId::new("pre", "1.2.3-alpha.0", sid).unwrap();
+        assert!(PackageIdSpec::parse("pre").unwrap().matches(pre));
+        assert!(!PackageIdSpec::parse("pre@1").unwrap().matches(pre));
+        assert!(!PackageIdSpec::parse("pre@1.2").unwrap().matches(pre));
+        assert!(!PackageIdSpec::parse("pre@1.2.3").unwrap().matches(pre));
+        assert!(PackageIdSpec::parse("pre@1.2.3-alpha.0")
+            .unwrap()
+            .matches(pre));
+        assert!(!PackageIdSpec::parse("pre@1.2.3-alpha.1")
+            .unwrap()
+            .matches(pre));
+        assert!(!PackageIdSpec::parse("pre@1.2.3-beta.0")
+            .unwrap()
+            .matches(pre));
+        assert!(!PackageIdSpec::parse("pre@1.2.3+hello")
+            .unwrap()
+            .matches(pre));
+        assert!(!PackageIdSpec::parse("pre@1.2.3-alpha.0+hello")
+            .unwrap()
+            .matches(pre));
     }
 }
